searchData={"items":[{"type":"behaviour","title":"Absinthe.Phoenix.Controller","doc":"Supports use of GraphQL documents inside Phoenix controllers.","ref":"Absinthe.Phoenix.Controller.html"},{"type":"behaviour","title":"Example - Absinthe.Phoenix.Controller","doc":"First, `use Absinthe.Phoenix.Controller`, passing your `schema` and\nnotifying Absinthe to operate in `internal` mode:\n\n```elixir\ndefmodule MyAppWeb.UserController do\n  use MyAppWeb, :controller\n  use Absinthe.Phoenix.Controller, schema: MyAppWeb.Schema, action: [mode: :internal]\n\n  # ... actions\n\nend\n```\n\nFor each action you want Absinthe to process, provide a GraphQL document using\nthe `@graphql` module attribute (before the action):\n\n```\n@graphql \"\"\"\n  query ($filter: UserFilter) {\n    users(filter: $filter, limit: 10)\n  }\n\"\"\"\ndef index(conn, %{data: data}) do\n  render conn, \"index.html\", data\nend\n```\n\nThe params for the action will be intercepted by the\n`Absinthe.Phoenix.Controller.Action` plug, and used as variables for\nthe GraphQL document you've specified.\n\nFor instance, given a definition for a `:user_filter` input object\ntype like this:\n\n```\ninput_object :user_filter do\n  field :name_matches, :string\n  field :age_above, :integer\n  field :age_below, :integer\nend\n```\n\nAnd a query that looks like this (assuming you have the normal\n`Plug.Parsers` configuration for param parsing):\n\n```\n?filter[name_matches]=joe&filter[age_above]=42\n```\n\nThen Absinthe will receive variable definitions of:\n\n```\n%{\"filter\" => %{\"name_matches\" => \"joe\", \"age_above\" => 42}}\n```\n\n(For how the string `\"42\"` was converted into `42`, see `cast_param/3`).\n\nThe params on the `conn` will then be replaced by the result of the\nexecution by Absinthe. The action function can then match against\nthat result to respond correctly to the user:\n\nIt's up to you to handle the three possible results:\n\n- When there's `:data` but no `:errors`, everything went perfectly.\n- When there's `:errors` but no `:data`, a validation error occurred and the document could not be\n  executed.\n- When there's `:data` and `:errors`, partial data is available but some fields reported errors\n  during execution.\n\nNotice the keys are atoms, not strings as in normal Phoenix action invocations.","ref":"Absinthe.Phoenix.Controller.html#module-example"},{"type":"behaviour","title":"Differences with the GraphQL Specification - Absinthe.Phoenix.Controller","doc":"There are some important differences between GraphQL documents as\nprocessed in an HTTP API and the GraphQL documents that this module\nsupports.\n\nIn an effort to make use of GraphQL ergonomic in Phoenix controllers\nand views, Absinthe supports some slight structural modifications to\nthe GraphQL documents provided using the `@graphql` module attribute\nin controller modules.\n\nIn a way, you can think of these changes as a specialized GraphQL\ndialect. The following are the differences you need to keep in mind.\n\n#","ref":"Absinthe.Phoenix.Controller.html#module-differences-with-the-graphql-specification"},{"type":"behaviour","title":"Objects can be leaf nodes - Absinthe.Phoenix.Controller","doc":"Let's look at the `users` example mentioned before:\n\n```\n@graphql \"\"\"\n  query ($filter: UserFilter) {\n    users(filter: $filter, limit: 10)\n  }\n\"\"\"\n```\n\nYou'll notice that in the above example, `users` doesn't have an\naccompanying _selection set_ (that is, a set of child fields bounded\nby `{ ... }`). The GraphQL specification dictates that only scalar\nvalues can be \"leaf nodes\" in a GraphQL document... but to support\nunmodified struct values being returned (for example, Ecto schemas),\nif no selection set is provided for an object value (or list\nthereof), the entire value is returned.\n\nThe template can then use `users` as needed:\n\n```\n \n  <%= for user <- @users do %>\n     <%= link user.full_name, to: user_path(@conn, :show, user) %> \n  <% end %>\n \n```\n\nThis is useful for `Phoenix.HTML` helper functions that expect\nstructs with specific fields (especially `form_for`).\n\nOne way to think of this change is that, for objects, no selection\nset is equivalent to a \"splat\" operator (except, of course, even\nfields not defined in your GraphQL schema are returned as part of\nthe value).\n\nBut, never fear, nothing is stopping you from ignoring this behavior\nand providing a selection set if you want a traditionally narrow set\nof fields:\n\n```\n@graphql \"\"\"\n  query ($filter: UserFilter) {\n    users(filter: $filter, limit: 10) {\n      id\n      full_name\n    }\n  }\n\"\"\"\n```\n\n#","ref":"Absinthe.Phoenix.Controller.html#module-objects-can-be-leaf-nodes"},{"type":"behaviour","title":"Scalar values aren't serialized - Absinthe.Phoenix.Controller","doc":"To remove the need for reparsing values, scalar values aren't serialized;\nPhoenix actions receive the original, unserialized values of GraphQL fields.\n\nThis is especially useful for custom scalar types. Using a couple of the\nadditional types packaged in `Absinthe.Type.Custom`, for example:\n\n- `:decimal` values are returned as `%Decimal{}` structs, not strings.\n- `:datetime` values are returned as `%DateTime{}` structs, not strings.\n\nIn short, GraphQL used in controllers is a query language to retrieve the values requested---there's no need to serialize the\nvalues to send them across HTTP.\n\n#","ref":"Absinthe.Phoenix.Controller.html#module-scalar-values-aren-t-serialized"},{"type":"behaviour","title":"Fields use snake_case - Absinthe.Phoenix.Controller","doc":"Unlike in the GraphQL notation scheme we prefer for GraphQL APIs (that is,\n`camelCase` fields, which better match up with the expectations of JavaScript\nclients), fields used in documents provided as `@graphql` should use\n`snake_case` naming, as Elixir conventions use that notation style for atoms,\netc.\n\n#","ref":"Absinthe.Phoenix.Controller.html#module-fields-use-snake_case"},{"type":"behaviour","title":"Atom keys - Absinthe.Phoenix.Controller","doc":"Because you are writing the GraphQL document in your controller and Absinthe\nis validating the document against your schema, atom keys are returned for\nfield names.","ref":"Absinthe.Phoenix.Controller.html#module-atom-keys"},{"type":"callback","title":"Absinthe.Phoenix.Controller.absinthe_pipeline/2","doc":"Customize the Absinthe processing pipeline.\n\nOnly implement this function if you need to change the pipeline used\nto process documents.","ref":"Absinthe.Phoenix.Controller.html#c:absinthe_pipeline/2"},{"type":"callback","title":"Absinthe.Phoenix.Controller.cast_param/3","doc":"Cast string param values to values Absinthe expects for variable input.\n\nSome scalar types, like `:integer` (GraphQL `Int`) require that raw,\nincoming value be a non-string type. This isn't a problem in\nGraphQL-over-HTTP because the variable values are provided as a JSON\npayload (which supports, i.e., integer values).\n\nTo support converting incoming param values to the format that\ncertain scalars expect, we support a `cast_param/3` callback\nfunction that takes a raw value, target type (e.g., the scalar\ntype), and the schema, and returns the transformed\nvalue. `cast_param/3` is overridable and the implementation already\nsupports `:integer` and `:float` types.\n\nIf you override `cast_param/3`, make sure you super or handle lists,\nnon-nulls, and input object values yourself; they're also processed\nusing the function.\n\nImportant: In the event that a value is _invalid_, just return it\nunchanged so that Absinthe's usual validation logic can report it as\ninvalid.","ref":"Absinthe.Phoenix.Controller.html#c:cast_param/3"},{"type":"function","title":"Absinthe.Phoenix.Controller.default_pipeline/2","doc":"","ref":"Absinthe.Phoenix.Controller.html#default_pipeline/2"},{"type":"function","title":"Absinthe.Phoenix.Controller.variables/1","doc":"","ref":"Absinthe.Phoenix.Controller.html#variables/1"},{"type":"module","title":"Absinthe.Phoenix.Endpoint","doc":"","ref":"Absinthe.Phoenix.Endpoint.html"},{"type":"module","title":"Absinthe.Phoenix.Socket","doc":"`Absinthe.Phoenix.Socket` is used as a module for setting up a control\nchannel for handling GraphQL subscriptions.","ref":"Absinthe.Phoenix.Socket.html"},{"type":"module","title":"Examples - Absinthe.Phoenix.Socket","doc":"defmodule MyApp.Web.UserSocket do\n      use Phoenix.Socket\n      use Absinthe.Phoenix.Socket,\n        schema: MyApp.Web.Schema\n\n      transport :websocket, Phoenix.Transports.WebSocket\n\n      def connect(params, socket) do\n        socket = Absinthe.Phoenix.Socket.put_options(socket, [\n          context: %{current_user: find_current_user(params)}\n        ])\n        {:ok, socket}\n      end\n\n      def id(_socket), do: nil\n    end","ref":"Absinthe.Phoenix.Socket.html#module-examples"},{"type":"module","title":"Phoenix 1.2 - Absinthe.Phoenix.Socket","doc":"If you're on Phoenix 1.2 see `put_schema/2`.","ref":"Absinthe.Phoenix.Socket.html#module-phoenix-1-2"},{"type":"module","title":"Garbage Collection - Absinthe.Phoenix.Socket","doc":"In some workloads, the Channel Process responsible for handling [subscriptions may accumulate\nsome gargage](https://elixirforum.com/t/why-does-garbage-collection-not-work-as-intended/50613/2), that is not being collected by the Erlang VM. A workaround for this is to instruct\nthe process to periodically tell the VM to collect its garbage. This can be done by setting the\n`gc_interval`.\n\n      use Absinthe.Phoenix.Socket,\n        schema: MyApp.Web.Schema,\n        gc_interval: 10_000","ref":"Absinthe.Phoenix.Socket.html#module-garbage-collection"},{"type":"function","title":"Absinthe.Phoenix.Socket.put_options/2","doc":"Configure Absinthe options for a socket.","ref":"Absinthe.Phoenix.Socket.html#put_options/2"},{"type":"function","title":"Examples - Absinthe.Phoenix.Socket.put_options/2","doc":"def connect(params, socket) do\n      current_user = current_user(params)\n      socket = Absinthe.Phoenix.Socket.put_options(socket, context: %{\n        current_user: current_user\n      })\n      {:ok, socket}\n    end\n\n    defp current_user(%{\"user_id\" => id}) do\n      MyApp.Repo.get(User, id)\n    end","ref":"Absinthe.Phoenix.Socket.html#put_options/2-examples"},{"type":"function","title":"Absinthe.Phoenix.Socket.put_schema/2","doc":"Configure the schema for a socket.\n\nOnly use this if you are not yet on Phoenix 1.3. If you're on Phoenix 1.3,\nread the moduledocs.","ref":"Absinthe.Phoenix.Socket.html#put_schema/2"},{"type":"module","title":"Absinthe.Phoenix.SubscriptionTest","doc":"Convenience functions for subscription tests.","ref":"Absinthe.Phoenix.SubscriptionTest.html"},{"type":"function","title":"Absinthe.Phoenix.SubscriptionTest.join_absinthe/1","doc":"","ref":"Absinthe.Phoenix.SubscriptionTest.html#join_absinthe/1"},{"type":"function","title":"Absinthe.Phoenix.SubscriptionTest.push_doc/3","doc":"A small wrapper around `Phoenix.ChannelTest.push/3`.\n\nThe only option that is used is `opts[:variables]` - all other options are\nignored.","ref":"Absinthe.Phoenix.SubscriptionTest.html#push_doc/3"},{"type":"module","title":"Absinthe.Phoenix.Types","doc":"","ref":"Absinthe.Phoenix.Types.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v2.0.0 - 2020-05-14 - Changelog","doc":"- Phoenix.PubSub 2.0 support","ref":"changelog.html#v2-0-0-2020-05-14"},{"type":"extras","title":"v1.5.0 - 2020-05-14 - Changelog","doc":"- Absinthe 1.5 support","ref":"changelog.html#v1-5-0-2020-05-14"},{"type":"extras","title":"v1.4.4 - 2019-05-04 - Changelog","doc":"- Remove unintentional poison dependency","ref":"changelog.html#v1-4-4-2019-05-04"},{"type":"extras","title":"v1.4.2 - 2018-01-22 - Changelog","doc":"- Add `variables/1` function to make it easy to get the original parameters","ref":"changelog.html#v1-4-2-2018-01-22"},{"type":"extras","title":"v1.4.1 - 2018-01-21 - Changelog","doc":"- Deprecate `Absinthe.Phoenix.Socket.put_opts/2` in favor of `Absinthe.Phoenix.Socket.put_options/2` for consistency with Absinthe.Plug","ref":"changelog.html#v1-4-1-2018-01-21"},{"type":"extras","title":"v1.4.0 - 2017-11-13 - Changelog","doc":"- First real release.","ref":"changelog.html#v1-4-0-2017-11-13"},{"type":"extras","title":"Contributing","doc":"# Contributing\n\nThank you for contributing to the project. We'd love to see your\nissues and pull requests.\n\nIf you're creating a pull request, please consider these suggestions:\n\nFork, then clone the repo:\n\n    git clone git@github.com:your-username/absinthe_phoenix.git\n\nInstall the dependencies:\n\n    mix deps.get\n\nMake sure the tests pass:\n\n    mix test\n\nMake your change. Add tests for your change. Make the tests pass:\n\n    mix test\n\nPush to your fork (preferably to a non-`master` branch) and\n[submit a pull request][pr].\n\n[pr]: https://github.com/absinthe-graphql/absinthe_phoenix/compare/\n\nWe'll review and answer your pull request as soon as possible. We may suggest\nsome changes, improvements, or alternatives. Let's work through it together.\n\nSome things that will increase the chance that your pull request is accepted:\n\n* Write tests.\n* Include `@typedoc`s, `@spec`s, and `@doc`s\n* Try to match the style conventions already present (and Elixir conventions,\n  generally).\n* Write a [good commit message][commit].\n\nThanks again for helping!\n\n[commit]: http://tbaggery.com/2008/04/19/a-note-about-git-commit-messages.html","ref":"contributing.html"},{"type":"extras","title":"Code of Conduct","doc":"# Code of Conduct\n\nContact: conduct@absinthe-graphql.org","ref":"code_of_conduct.html"},{"type":"extras","title":"Why have a Code of Conduct? - Code of Conduct","doc":"As contributors and maintainers of this project, we are committed to providing a friendly, safe and welcoming environment for all, regardless of age, disability, gender, nationality, race, religion, sexuality, or similar personal characteristic.\n\nThe goal of the Code of Conduct is to specify a baseline standard of behavior so that people with different social values and communication styles can talk about the Absinthe project effectively, productively, and respectfully, even in face of disagreements. The Code of Conduct also provides a mechanism for resolving conflicts in the community when they arise.","ref":"code_of_conduct.html#why-have-a-code-of-conduct"},{"type":"extras","title":"Our Values - Code of Conduct","doc":"These are the values developers that participate in Absinthe project-related activities should aspire to:\n\n  * Be friendly and welcoming\n  * Be patient\n    * Remember that people have varying communication styles and that not everyone is using their native language. (Meaning and tone can be lost in translation.)\n  * Be thoughtful\n    * Productive communication requires effort. Think about how your words will be interpreted.\n    * Remember that sometimes it is best to refrain entirely from commenting.\n  * Be respectful\n    * In particular, respect differences of opinion. It is important that we resolve disagreements and differing views constructively.\n  * Avoid destructive behavior\n    * Derailing: stay on topic; if you want to talk about something else, start a new conversation.\n    * Unconstructive criticism: don't merely decry the current state of affairs; offer (or at least solicit) suggestions as to how things may be improved.\n    * Snarking (pithy, unproductive, sniping comments).\n\nThe following actions are explicitly forbidden:\n\n  * Insulting, demeaning, hateful, or threatening remarks.\n  * Discrimination based on age, disability, gender, nationality, race, religion, sexuality, or similar personal characteristic.\n  * Bullying or systematic harassment.\n  * Unwelcome sexual advances.\n  * Incitement to any of these.","ref":"code_of_conduct.html#our-values"},{"type":"extras","title":"Where does the Code of Conduct apply? - Code of Conduct","doc":"If you participate in or contribute to the Absinthe project ecosystem in any way, you are encouraged to follow the Code of Conduct while doing so.\n\nExplicit enforcement of the Code of Conduct applies to the official mediums operated by the Absinthe project:\n\n* The official GitHub projects and code reviews.\n* The absinthe-graphql.org website and documentation.\n* The #absinthe-graphql Slack channel.\n\nActivities related to Absinthe (conference talks, meetups, and other unofficial forums) are encouraged to adopt this Code of Conduct. Such groups must provide their own contact information.\n\nProject maintainers may remove, edit, or reject comments, commits, code, wiki edits, issues, and other contributions that are not aligned to this Code of Conduct.\n\nInstances of abusive, harassing, or otherwise unacceptable behavior may be reported by emailing: conduct@absinthe-graphql.org. All complaints will be reviewed and investigated and will result in a response that is deemed necessary and appropriate to the circumstances. **All reports will be kept confidential**.\n\n**The goal of the Code of Conduct is to resolve conflicts in the most harmonious way possible**. We hope that in most cases issues may be resolved through polite discussion and mutual agreement. Bannings and other forceful measures are to be employed only as a last resort. **Do not** post about the issue publicly or try to rally sentiment against a particular individual or group.","ref":"code_of_conduct.html#where-does-the-code-of-conduct-apply"},{"type":"extras","title":"Acknowledgements - Code of Conduct","doc":"This document was based on the Code of Conduct from the Elixir project, which is based on the Code of Conduct from the Go project with parts derived from Django's Code of Conduct, Rust's Code of Conduct, and the Contributor Covenant.","ref":"code_of_conduct.html#acknowledgements"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2016 Bruce Williams, Ben Wilson\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# Absinthe.Phoenix\n\n[![Build Status](https://github.com/absinthe-graphql/absinthe_phoenix/workflows/CI/badge.svg)](https://github.com/absinthe-graphql/absinthe_phoenix/actions?query=workflow%3ACI)\n[![Version](https://img.shields.io/hexpm/v/absinthe_phoenix.svg)](https://hex.pm/packages/absinthe_phoenix)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/absinthe_phoenix/)\n[![Download](https://img.shields.io/hexpm/dt/absinthe_phoenix.svg)](https://hex.pm/packages/absinthe_phoenix)\n[![License](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)\n[![Last Updated](https://img.shields.io/github/last-commit/absinthe-graphql/absinthe_phoenix.svg)](https://github.com/absinthe-graphql/absinthe_phoenix/commits/master)\n\nThis package integrates Absinthe subscriptions with Phoenix, so that you can use subscriptions via websockets.\n\nFor getting started guides on subscriptions see: https://hexdocs.pm/absinthe/subscriptions.html\n\nFor getting started guides on server side rendering see: https://hexdocs.pm/absinthe/subscriptions.html","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"Install from [Hex.pm](https://hex.pm/packages/absinthe_phoenix):\n\nFor Phoenix 1.4, see the v1.5 branch: https://github.com/absinthe-graphql/absinthe_phoenix/tree/v1.5\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Phoenix 1.5 - Overview","doc":"```elixir\ndef deps do\n  [\n    {:absinthe_phoenix, \"~> 2.0.0\"}\n  ]\nend\n```\n\nNote: Absinthe.Phoenix requires Elixir 1.10 or higher.","ref":"readme.html#phoenix-1-5"},{"type":"extras","title":"Upgrading - Overview","doc":"See [CHANGELOG](./CHANGELOG.md) for upgrade steps between versions.\n\nYou may want to look for the specific upgrade guide in the [Absinthe documentation](https://hexdocs.pm/absinthe).","ref":"readme.html#upgrading"},{"type":"extras","title":"Documentation - Overview","doc":"See \"Usage,\" below, for basic usage information and links to specific resources.\n\n- [Absinthe.Phoenix hexdocs](https://hexdocs.pm/absinthe_phoenix).\n- For the tutorial, guides, and general information about Absinthe-related\n  projects, see [http://absinthe-graphql.org](http://absinthe-graphql.org).","ref":"readme.html#documentation"},{"type":"extras","title":"Related Projects - Overview","doc":"See the [GitHub organization](https://github.com/absinthe-graphql).","ref":"readme.html#related-projects"},{"type":"extras","title":"Usage - Overview","doc":"You need to have a working Phoenix PubSub configured. Here is what the default looks like if you create a new Phoenix project:\n\n```elixir\nconfig :my_app, MyAppWeb.Endpoint,\n  # ... other config\n  pubsub_server: MyApp.PubSub\n```\n\nIn your application supervisor add a line AFTER your existing endpoint supervision\nline:\n\n```elixir\n[\n  # other children ...\n  MyAppWeb.Endpoint, # this line should already exist\n  {Absinthe.Subscription, MyAppWeb.Endpoint}, # add this line\n  # other children ...\n]\n```\n\nWhere `MyAppWeb.Endpoint` is the name of your application's phoenix endpoint.\n\nIn your `MyAppWeb.Endpoint` module add:\n\n```elixir\nuse Absinthe.Phoenix.Endpoint\n```\n\nIn your socket add:\n\n```elixir\nuse Absinthe.Phoenix.Socket,\n  schema: MyAppWeb.Schema\n```\n\nWhere `MyAppWeb.Schema` is the name of your Absinthe schema module.\n\nThat is all that's required for setup on the server.\n\nFor client side guidance see the guides.","ref":"readme.html#usage"},{"type":"extras","title":"GraphiQL Usage - Overview","doc":"From within GraphiQL:\nTo use Absinthe.Phoenix from within GraphiQL, you need to tell GraphiQL about your websocket endpoint.\n\n```elixir\nforward \"/graphiql\", Absinthe.Plug.GraphiQL,\n  schema: MyAppWeb.Schema,\n  socket: MyAppWeb.UserSocket\n```","ref":"readme.html#graphiql-usage"},{"type":"extras","title":"Community - Overview","doc":"The project is under constant improvement by a growing list of\ncontributors, and your feedback is important. Please join us in Slack\n(`#absinthe-graphql` under the Elixir Slack account) or the Elixir Forum\n(tagged `absinthe`).\n\nPlease remember that all interactions in our official spaces follow\nour [Code of Conduct](./CODE_OF_CONDUCT.md).","ref":"readme.html#community"},{"type":"extras","title":"Contributing - Overview","doc":"Please follow [contribution guide](./CONTRIBUTING.md).","ref":"readme.html#contributing"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2016 Bruce Williams, Ben Wilson\n\nReleased under the MIT License, which can be found in [LICENSE.md](./LICENSE.md).","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}